﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Infragistics.Controls.Charts;

namespace xamDataChartAddDataPoints
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            this.DataContext = new WindowViewModel();      
        }

        private void ScatterSeries_Loaded(object sender, RoutedEventArgs e)
        {
            Dispatcher.BeginInvoke(new Action(() => 
            {
                //Horizontal line binding
                Binding hb1 = new Binding("ActualHeight");
                hb1.ElementName = "ScatterLineYAxis";
                Binding hb2 = new Binding();
                hb2.ElementName = "ScatterLineYAxis";

                MultiBinding hmb = new MultiBinding();
                hmb.Converter = new VerticalConverter();
                hmb.Bindings.Add(hb1);
                hmb.Bindings.Add(hb2);
                horizontal.SetBinding(ValueOverlay.ValueProperty, hmb);

                //Vertical line binding
                Binding vb1 = new Binding("ActualWidth");
                vb1.ElementName = "ScatterLineXAxis";
                Binding vb2 = new Binding();
                vb2.ElementName = "ScatterLineXAxis";

                MultiBinding vmb = new MultiBinding();
                vmb.Converter = new HorizontalConverter();
                vmb.Bindings.Add(vb1);
                vmb.Bindings.Add(vb2);
                vertical.SetBinding(ValueOverlay.ValueProperty, vmb);
            }), System.Windows.Threading.DispatcherPriority.Background, null);            
        }
    }

    public class WindowViewModel
    {
        private ObservableCollection<LiveViewDataPoint> _chartSource = null;

        public WindowViewModel()
        {
            _chartSource = new ObservableCollection<LiveViewDataPoint>();

            _chartSource.Add(new LiveViewDataPoint() { Date = 12, Price = 13.68 });
            _chartSource.Add(new LiveViewDataPoint() { Date = 1, Price = -2.68 });
            _chartSource.Add(new LiveViewDataPoint() { Date = -6, Price = 1.56 });
            _chartSource.Add(new LiveViewDataPoint() { Date = -3, Price = 9.68 });
        }
        public ObservableCollection<LiveViewDataPoint> ChartSource
        {
            get { return _chartSource; }
        }
        public double xAxisMinimum
        {
            get
            {
                if (_chartSource == null)
                    return 0;
                double minDate = _chartSource.Min(m => m.Date);
                double minPrice = _chartSource.Min(m => m.Price);
                if (minDate > minPrice)
                    minDate = minPrice;
                if(minDate < 0)
                    return Math.Round((minDate + minDate / 10), 2);
                else
                    return Math.Round((minDate - minDate / 10), 2);
            }
        }

        public double xAxisMaximum
        {
            get
            {
                if (_chartSource == null)
                    return 0;
                double minDate = _chartSource.Max(m => m.Date);
                double minPrice = _chartSource.Max(m => m.Price);
                if (minDate < minPrice)
                    minDate = minPrice;
                return Math.Round((minDate + minDate / 10), 2);
            }
        }

        public double xAxisInterval
        {
            get
            {
                return (xAxisMaximum - xAxisMinimum) / 7;
            }
        }

        public double yAxisMinimum
        {
            get
            {
                if (_chartSource == null)
                    return 0;
                double minDate = _chartSource.Min(m => m.Date);
                double minPrice = _chartSource.Min(m => m.Price);
                if (minDate < minPrice)
                     minPrice = minDate;
                if(minPrice < 0)
                    return Math.Round((minPrice + minPrice / 10), 2);
                else
                    return Math.Round((minPrice - minPrice / 10), 2);
            }
        }

        public double yAxisMaximum
        {
            get
            {
                if (_chartSource == null)
                    return 0;
                double minDate = _chartSource.Max(m => m.Date);
                double minPrice = _chartSource.Max(m => m.Price);
                if (minDate > minPrice)
                     minPrice = minDate;
                return Math.Round((minPrice + minPrice / 10), 2);
            }
        }

        public double yAxisInterval
        {
            get
            {
                return (yAxisMaximum - yAxisMinimum) / 7;
            }
        }
    }

    public class LiveViewDataPoint
    {
        public int Date { get; set; }
        public double Price { get; set; }
    }

    public class ColorConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if ((double)value == 0)
            {
                return 1;
            }
            return .2;
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    public class HorizontalConverter : IMultiValueConverter
    {
        public object Convert(object[] values, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if ((double)values[0] != 0)
            {
                ScalerParams sp = new ScalerParams((values[1] as NumericXAxis).SeriesViewer.WindowRect, (values[1] as NumericXAxis).SeriesViewer.ViewportRect, false);
                double tmp = (values[1] as NumericXAxis).GetUnscaledValue((double)values[0] / 2, sp);
                return tmp;
            }
            return Binding.DoNothing;
        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    public class VerticalConverter : IMultiValueConverter
    {
        public object Convert(object[] values, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if ((double)values[0] != 0)
            {
                ScalerParams sp = new ScalerParams((values[1] as NumericYAxis).SeriesViewer.WindowRect, (values[1] as NumericYAxis).SeriesViewer.ViewportRect, false);
                double tmp = (values[1] as NumericYAxis).GetUnscaledValue((double)values[0] / 2, sp);
                return tmp;
            }
            return Binding.DoNothing;
        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }


}
